import { Cursor, Line } from './Cursor.new';
import { ComponentPropsWithoutRef, ForwardRefExoticComponent, ReactElement, RefAttributes } from 'react';
/** https://github.com/Microsoft/TypeScript/issues/29729 */
export declare type LiteralUnion<T extends U, U = string> = T | (Pick<U, never> & {
    _?: never;
});
declare type Omit<T, K extends keyof T> = Pick<T, Exclude<keyof T, K>>;
declare type PartialBy<T, K extends keyof T> = Omit<T, K> & Partial<Pick<T, K>>;
export declare const BUILT_IN_COMMANDS: readonly ["bold", "italic", "strike-through", "h1", "h2", "h3", "h4", "h5", "h6", "unordered-list", "ordered-list", "code-block", "code-inline", "code", "link", "image", "block-quotes"];
export declare type CommandType = LiteralUnion<typeof BUILT_IN_COMMANDS[number], string>;
export declare type CommandHandlerContext = {
    textarea: HTMLTextAreaElement;
    cursor: Cursor;
    keyEvent?: KeyboardEvent;
    options: TextareaMarkdownOptions;
};
export declare type CommandHandler<TArgs extends unknown[] = any[]> = (context: CommandHandlerContext, ...args: Partial<TArgs>) => void;
export declare type CommandConfig<TType extends CommandType = CommandType> = {
    /** Handler function for custom commands */
    handler: CommandHandler;
    /** Shortcut combinations ([Mousetrap.js](https://craig.is/killing/mice)) */
    shortcut?: string | string[];
    /** Toggle key event prevent `default:false` */
    shortcutPreventDefault?: boolean;
    /** Built-in or custom command name */
    name: TType;
    /** Toggle command enabling */
    enable?: boolean;
};
export declare type PrefixWrappingConfig = {
    prefix: string | ((enteringLine: Line) => string);
    prefixPattern?: RegExp;
    shouldBreakIfEmpty?: boolean;
    shouldSaveIndent?: boolean;
};
export declare type TextareaMarkdownOptions = {
    /** Preferred unordered list prefix `default: '-'` */
    preferredUnorderedListSyntax: '-' | '*' | '+';
    /** Preferred bold wrap syntax `default: '**'` */
    preferredBoldSyntax: '**' | '__';
    /** Preferred italic wrap syntax `default: '*'` */
    preferredItalicSyntax: '*' | '_';
    /** Will handle `tab`/`shift+tab` keystrokes, on which will insert/remove indentation instead of the default behavior `default:true` */
    enableIndentExtension: boolean;
    /** Will handle `command/ctrl+backspace` keystrokes, on which will remove only a current line instead of the default behavior `default:true` */
    enableProperLineRemoveBehaviorExtension: boolean;
    /** Will handle `paste` event, on which will wrap pasted with link or image markup if pasted is URL `default:true` */
    enableLinkPasteExtension: boolean;
    /** Will handle `enter` keystroke, on which will wrap current list sequence to a new line if needed `default:true` */
    enablePrefixWrappingExtension: boolean;
    /**
     * Will handle `enter` keystroke, on which will autocorrect invalid ordered list patterns `default:false`
     * @example
     * 1) some text -> 1. some text
     * 1.1) some text -> 1.1. some text
     * */
    enableOrderedListAutoCorrectExtension: boolean;
    /** Array of custom prefixes, that need to be wrapped. (Will not work with `enablePrefixWrappingExtension:false`) */
    customPrefixWrapping: (PrefixWrappingConfig | string)[];
    /** `default: 'bold'` */
    boldPlaceholder: string;
    /** `default: 'italic'` */
    italicPlaceholder: string;
    /** `default: 'strike through'` */
    strikeThroughPlaceholder: string;
    /** `default: 'code'` */
    codeInlinePlaceholder: string;
    /** `default: 'code block'` */
    codeBlockPlaceholder: string;
    /** `default: 'ordered list'` */
    orderedListPlaceholder: string;
    /** `default: 'unordered list'` */
    unorderedListPlaceholder: string;
    /** `default: (lvl) => 'headline ' + lvl` */
    headlinePlaceholder: string | ((level: number) => string);
    /** `default: 'quote'` */
    blockQuotesPlaceholder: string;
    /** Used inside default link markup `[<example>](...)`  `default: 'example'` */
    linkTextPlaceholder: string;
    /** Used inside default image markup `![...](<url>)`  `default: 'url'` */
    linkUrlPlaceholder: string;
    /** Used inside default image markup `![<example>](...)`  `default: 'example'` */
    imageTextPlaceholder: string;
    /** Used inside default image markup `![...](<image.png>)`  `default: 'image.png'` */
    imageUrlPlaceholder: string;
};
export declare const defaultTextareaMarkdownOptions: TextareaMarkdownOptions;
export declare type CommandTrigger = <TArgs extends unknown[] = unknown[]>(command: CommandType, ...args: TArgs) => void;
export declare type Command = PartialBy<CommandConfig, 'handler'>;
export declare type Extension = (textarea: HTMLTextAreaElement, options: TextareaMarkdownOptions) => void | (() => void);
export declare type TextareaMarkdownConfig = {
    commands?: Command[];
    options?: Partial<TextareaMarkdownOptions>;
};
export declare type TextareaMarkdownRef = HTMLTextAreaElement & {
    trigger: CommandTrigger;
    cursor: Cursor;
};
export declare type TextareaMarkdownProps = Omit<TextareaMarkdownConfig & ComponentPropsWithoutRef<'textarea'>, 'children'>;
export interface TextareaMarkdownComponent extends ForwardRefExoticComponent<TextareaMarkdownProps & RefAttributes<TextareaMarkdownRef>> {
    Wrapper: ForwardRefExoticComponent<TextareaMarkdownConfig & RefAttributes<TextareaMarkdownRef> & {
        children: ReactElement;
    }>;
}
export {};
